import logging
import runpy
import tempfile
import urllib.parse

import mkdocs.config
import mkdocs.config.config_options
import mkdocs.plugins
import mkdocs.structure.files

try:
    from mkdocs.exceptions import PluginError
except ImportError:
    PluginError = SystemExit

from . import editor

log = logging.getLogger(f"mkdocs.plugins.{__name__}")
log.addFilter(mkdocs.utils.warning_filter)


class GenFilesPlugin(mkdocs.plugins.BasePlugin):
    config_scheme = (("scripts", mkdocs.config.config_options.Type(list)),)

    def on_files(
        self, files: mkdocs.structure.files.Files, config: mkdocs.config.Config
    ) -> mkdocs.structure.files.Files:
        self._dir = tempfile.TemporaryDirectory(prefix="mkdocs_gen_files_")

        with editor.FilesEditor(files, config, self._dir.name) as ed:
            for file_name in self.config["scripts"]:
                try:
                    runpy.run_path(file_name)
                except SystemExit as e:
                    if e.code:
                        raise PluginError(f"Script {file_name!r} caused {e!r}")

        self._edit_paths = ed.edit_paths
        return ed.files

    def on_page_content(
        self,
        html,
        page: mkdocs.structure.pages.Page,
        config: mkdocs.config.Config,
        files: mkdocs.structure.files.Files,
    ):
        repo_url = config.get("repo_url", None)
        edit_uri = config.get("edit_uri", None)

        if page.file.src_path in self._edit_paths:
            path = self._edit_paths.pop(page.file.src_path)
            if repo_url and edit_uri:
                page.edit_url = path and urllib.parse.urljoin(
                    urllib.parse.urljoin(repo_url, edit_uri), path
                )

        return html

    def on_post_build(self, config: mkdocs.config.Config):
        self._dir.cleanup()

        if self._edit_paths:
            msg = "mkdocs_gen_files: These set_edit_path invocations went unused (the files don't exist): %r"
            log.warning(msg, self._edit_paths)



import collections
import os
import os.path
import pathlib
import shutil
from typing import IO, ClassVar, Mapping, Optional

import mkdocs.config
import mkdocs.structure.files


def _normpath(*path: str):
    return os.path.normpath(os.path.join(*path)).replace(os.sep, "/")


def _file_sort_key(f: mkdocs.structure.files.File):
    parts = pathlib.PurePath(f.src_path).parts
    return tuple(chr(i != len(parts) - 1) + chr(f.name != "index") + p for i, p in enumerate(parts))


class FilesEditor:
    config: mkdocs.config.Config = None
    """The current MkDocs [config](https://www.mkdocs.org/user-guide/plugins/#config)."""
    directory: str = None
    """The base directory for `open()` ([docs_dir](https://www.mkdocs.org/user-guide/configuration/#docs_dir))."""
    edit_paths: Mapping[str, Optional[str]]

    def open(self, name: str, mode, buffering=-1, encoding=None, *args, **kwargs) -> IO:
        """Open a file under `docs_dir` virtually.

        This function, for all intents and purposes, is just an `open()` which pretends that it is
        running under [docs_dir](https://www.mkdocs.org/user-guide/configuration/#docs_dir)
        (*docs/* by default), but write operations don't affect the actual files when running as
        part of a MkDocs build, but they do become part of the site build.
        """
        path = self._get_file(name, new="w" in mode)
        if encoding is None and "b" not in mode:
            encoding = "utf-8"
        return open(path, mode, buffering, encoding, *args, **kwargs)

    def _get_file(self, name: str, new: bool = False) -> str:
        new_f = mkdocs.structure.files.File(
            name,
            src_dir=self.directory,
            dest_dir=self.config["site_dir"],
            use_directory_urls=self.config["use_directory_urls"],
        )
        normname = _normpath(name)

        if new or normname not in self._files:
            os.makedirs(os.path.dirname(new_f.abs_src_path), exist_ok=True)
            self._files[normname] = new_f
            self.edit_paths.setdefault(normname, None)
            return new_f.abs_src_path

        f = self._files[normname]
        if f.abs_src_path != new_f.abs_src_path:
            os.makedirs(os.path.dirname(new_f.abs_src_path), exist_ok=True)
            self._files[normname] = new_f
            self.edit_paths.setdefault(normname, None)
            shutil.copyfile(f.abs_src_path, new_f.abs_src_path)
            return new_f.abs_src_path

        return f.abs_src_path

    def set_edit_path(self, name: str, edit_name: Optional[str]) -> None:
        """Choose a file path to use for the edit URI of this file."""
        self.edit_paths[_normpath(name)] = edit_name and str(edit_name)

    def __init__(
        self,
        files: mkdocs.structure.files.Files,
        config: mkdocs.config.Config,
        directory: Optional[str] = None,
    ):
        self._files = collections.ChainMap({}, {_normpath(f.src_path): f for f in files})
        self.config = config
        if directory is None:
            directory = config["docs_dir"]
        self.directory = directory
        self.edit_paths = {}

    _current: ClassVar[Optional["FilesEditor"]] = None
    _default: ClassVar[Optional["FilesEditor"]] = None

    @classmethod
    def current(cls) -> "FilesEditor":
        """The instance of FilesEditor associated with the currently ongoing MkDocs build.

        If used as part of a MkDocs build (*gen-files* plugin), it's an instance using virtual
        files that feed back into the build.

        If not, this still tries to load the MkDocs config to find out the *docs_dir*, and then
        actually performs any file writes that happen via `.open()`.

        This is global (not thread-safe).
        """
        if cls._current:
            return cls._current
        if not cls._default:
            config = mkdocs.config.load_config("mkdocs.yml")
            config["plugins"].run_event("config", config)
            cls._default = FilesEditor(mkdocs.structure.files.Files([]), config)
        return cls._default

    def __enter__(self):
        type(self)._current = self
        return self

    def __exit__(self, *exc):
        type(self)._current = None

    @property
    def files(self) -> mkdocs.structure.files.Files:
        """Access the files as they currently are, as a MkDocs [Files][] collection.

        [Files]: https://github.com/mkdocs/mkdocs/blob/master/mkdocs/structure/files.py
        """
        files = sorted(self._files.values(), key=_file_sort_key)
        return mkdocs.structure.files.Files(files)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mkdocs_gen_files']

package_data = \
{'': ['*']}

install_requires = \
['mkdocs>=1.0,<2.0']

entry_points = \
{'mkdocs.plugins': ['gen-files = mkdocs_gen_files.plugin:GenFilesPlugin']}

setup_kwargs = {
    'name': 'mkdocs-gen-files',
    'version': '0.3.0',
    'description': 'MkDocs plugin to programmatically generate documentation pages during the build',
    'long_description': None,
    'author': 'Oleh Prypin',
    'author_email': 'oleh@pryp.in',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/oprypin/mkdocs-gen-files',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
